<?php

namespace App\Form;

use App\Entity\Case\StandardNote;
use App\Enums\Case\NoteLocation;
use App\Enums\Case\NoteMethod;
use App\Enums\Case\NoteStatus;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\Extension\Core\Type\EnumType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;

class StandardNoteFormType extends AbstractType
{
    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $members = $options['members'];

        $ids = [];
        foreach ($members as $idx => $member) {
            $ids[$idx] = $member->getId()->toString();
        }

        $builder
            ->add('date', null, [
                'widget' => 'single_text',
            ])
            ->add('startTime', null, [
                'widget' => 'single_text',
            ])
            ->add('endTime', null, [
                'widget' => 'single_text',
            ])
            ->add('location', EnumType::class, [
                'class' => NoteLocation::class
            ])
            ->add('status', EnumType::class, [
                'class' => NoteStatus::class,
            ])
            ->add('note', null, [
                'required' => true,
                'attr' => [
                    'class' => 'form-control',
                    'placeholder' => 'Note',
                ],
            ])
            ->add('members', ChoiceType::class, [
                //'class' => Member::class,
                'choices' => $members,
                'choice_attr' => function ($member) {
                    return [
                        'checked' => $member->isChecked(),
                        'value' => $member->getId()->toString()
                    ];
                },
                'multiple' => true,
                'expanded' => true,
                'choice_label' => 'name',
                'mapped' => false,
            ])
            ->add('method', EnumType::class, [
                'class' => NoteMethod::class,
            ])
        ;
    }

    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([
            'data_class' => StandardNote::class,
            'members' => [],
            'csrf_protection' => true,
            'csrf_field_name' => '_token',
            'csrf_token_id' => 'standard_note',
        ]);
    }
}
